/*
 
 FSProcessIndicator.m
 
 Copyright (c) 2012 Florian Schliep <http://floschliep.com>. All rights reserved. Licensed under the BSD License <http://opensource.org/licenses/bsd-license>.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 * Neither the name of the author nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
*/

#import "FSProcessIndicator.h"

@implementation FSProcessIndicator

- (id)initWithFrame:(NSRect)frame {
    
    self = [super initWithFrame:frame];
    
    if (self) {
        
        fins = 12;
                
        colors = [[NSMutableArray alloc] initWithObjects:
                  [NSColor colorWithCalibratedWhite:1.0f alpha:1.0f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.95f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.85f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.75f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.65f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.55f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.45f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.35f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.25f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.2f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.15f],
                  [NSColor colorWithCalibratedWhite:1.0f alpha:0.1f],
                  nil];
        
    }
    
    return self;
}

- (void)drawRect:(NSRect)dirtyRect {
    
    CGContextRef currentContext = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];
    [NSGraphicsContext saveGraphicsState];
    
    CGContextTranslateCTM(currentContext, (dirtyRect.size.width / 2), (dirtyRect.size.height / 2));
    
    NSBezierPath *path = [[NSBezierPath alloc] init];
    [path setLineCapStyle:NSRoundLineCapStyle];
    [path setLineWidth:(.0859375 * dirtyRect.size.width)];
    [path moveToPoint:NSMakePoint(0, (.234375 * dirtyRect.size.width))];
    [path lineToPoint:NSMakePoint(0, (.421875 * dirtyRect.size.width))];
    
    for (int i = 0; i < fins; i++) {
        
        if (isAnimating) {
            
            NSColor *currentColor = [colors objectAtIndex:i];
            [currentColor set];
            
        } else {
            
            [[NSColor clearColor] set];
            
        }
        
        [path stroke];
        
        CGContextRotateCTM(currentContext, 6.282185/fins);
        
    }
    
    [NSGraphicsContext restoreGraphicsState];
    
}

- (void)updateIndicator {
   
    [self setFrameCenterRotation:(self.frameCenterRotation - 30)];
    [self setNeedsDisplay:YES];
        
}

- (void)startAnimation {
    
    isAnimating = YES;
    animTimer = [NSTimer scheduledTimerWithTimeInterval:0.05 target:self selector:@selector(updateIndicator) userInfo:nil repeats:YES];
    [animTimer fire];
    
}

- (void)stopAnimation {
    
    [animTimer invalidate];
    animTimer = nil;
    isAnimating = NO;
    
}

@end
