/*
 
 FSProcessView.h
 
 Copyright (c) 2012 Florian Schliep <http://floschliep.com>. All rights reserved. Licensed under the BSD License <http://opensource.org/licenses/bsd-license>.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 * Neither the name of the author nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
*/

#import <Cocoa/Cocoa.h>
#import "FSProcessIndicator.h"


@interface FSProcessView : NSView {
    
    FSProcessIndicator *indicator;
        
    SEL methodForExecution;
    id targetForExecution;
    id objectForExecution;
    
}

@property (nonatomic, strong) NSView *parentView;
/* The view in which the process view will be shown. */

@property (nonatomic) BOOL animates;
/* Whether the process view animates or not. */


+ (FSProcessView *)defaultProcessViewWithParentView:(NSView *)view;
/* Initializes a new process view and sets the view in which the process view will be shown. */

- (id)initWithParentView:(NSView *)view;
/* Init method to set the view in which the process view will be shown. */

- (void)show;
/* Method to show the process view. */

- (void)hide;
/* Method to hide the process view. */

- (void)showWhileExecuting:(SEL)method onTarget:(id)target withObject:(id)object;
/* Method to show the process view while executing a method on a target with an (optional) object. The process view will automatically hide. */

@end