/*
 
 FSProcessView.m
 
 Copyright (c) 2012 Florian Schliep <http://floschliep.com>. All rights reserved. Licensed under the BSD License <http://opensource.org/licenses/bsd-license>.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
 * Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 
 * Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
 
 * Neither the name of the author nor the names of its contributors may be used
 to endorse or promote products derived from this software without specific
 prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
*/

#import "FSProcessView.h"

@implementation FSProcessView

@synthesize parentView;
@synthesize animates;

- (id)initWithParentView:(NSView *)view {
    
    self = [super initWithFrame:NSMakeRect(0, 0, 64, 64)];
    
    if (self) {
        
        self.parentView = view;
        
        indicator = [[FSProcessIndicator alloc] initWithFrame:NSMakeRect(0, 0, 32, 32)];
        [indicator setAutoresizingMask:NSViewMinXMargin | NSViewMaxXMargin | NSViewMinYMargin | NSViewMaxYMargin];
        [indicator setFrameOrigin:NSMakePoint((NSWidth([self bounds]) - NSWidth([indicator bounds])) / 2, (NSHeight([self bounds]) - NSHeight([indicator bounds])) / 2)];
        [self addSubview:indicator];
                
        [self setAutoresizingMask:NSViewMinXMargin | NSViewMaxXMargin | NSViewMinYMargin | NSViewMaxYMargin];
        [self setFrameOrigin:NSMakePoint((NSWidth([self.parentView bounds]) - NSWidth([self bounds])) / 2, (NSHeight([self.parentView bounds]) - NSHeight([self bounds])) / 2)];
        
        [self.layer setOpaque:NO];
        [self setWantsLayer:YES];
        
    }
    
    return self;
}

+ (FSProcessView *)defaultProcessViewWithParentView:(NSView *)view {
    return [[FSProcessView alloc] initWithParentView:view];
}

- (void)drawRect:(NSRect)dirtyRect {
    
    [[NSBezierPath bezierPathWithRoundedRect:dirtyRect xRadius:(.175438596 * 64) yRadius:(.175438596 * 64)] addClip];
    [[NSColor colorWithCalibratedWhite:0.0f alpha:0.5f] set];
    NSRectFill(dirtyRect);
        
}

- (void)show {
    
    [indicator startAnimation];
    
    if (animates) {
     
        [self setAlphaValue:0.0f];
        
        [NSAnimationContext beginGrouping];
        [[NSAnimationContext currentContext] setDuration:.3f];
        
        [self.parentView.animator addSubview:self];
        [self.animator setAlphaValue:1.0f];
        
        [NSAnimationContext endGrouping];
        
    } else {
        
        [self.parentView addSubview:self];
        
    }
        
}

- (void)hide {
    
    if (animates) {
                
        [NSAnimationContext beginGrouping];
        [[NSAnimationContext currentContext] setDuration:.3f];
        
        [self.animator setAlphaValue:0.0f];
        
        [NSAnimationContext endGrouping];
        
        [self performSelector:@selector(removeFromSuperview) withObject:nil afterDelay:0.3f];
        [indicator performSelector:@selector(stopAnimation) withObject:nil afterDelay:0.3f];
        
    } else {
        
        [self removeFromSuperview];
        [indicator stopAnimation];
        
    }
    
}

- (void)showWhileExecuting:(SEL)method onTarget:(id)target withObject:(id)object {
    
    methodForExecution = method;
    targetForExecution = target;
    objectForExecution = object;
    
    [NSThread detachNewThreadSelector:@selector(launchExecution) toTarget:self withObject:nil];
    
    [self show];
    
}

- (void)launchExecution {
    @autoreleasepool {
        
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Warc-performSelector-leaks"
        [targetForExecution performSelector:methodForExecution withObject:objectForExecution];
        
#pragma clang diagnostic pop
        [self performSelectorOnMainThread:@selector(hide) withObject:nil waitUntilDone:NO];
        
    }
}

@end